package ap.proof.goal;

import ap.terfor.{Conjunction, ConstantTerm, TermOrder}

object UpdateGoalTaskResult {

  def apply(updatedFacts : Conjunction,
            updatedElimConstants : Set[ConstantTerm],
            updatedOrder : TermOrder,
            newTasks : Iterable[Task],
            goal : Goal) =
    new UpdateGoalTaskResult(updatedFacts, updatedElimConstants,
                             updatedOrder, newTasks)
         
  def apply(updatedElimConstants : Set[ConstantTerm],
            updatedOrder : TermOrder,
            newTasks : Iterable[Task],
            goal : Goal) =
    new UpdateGoalTaskResult(goal.facts, updatedElimConstants, updatedOrder, newTasks)
         
  def apply(updatedFacts : Conjunction,
            newTasks : Iterable[Task],
            goal : Goal) =
    new UpdateGoalTaskResult(updatedFacts, goal.eliminatedConstants, goal.order, newTasks)
    
  def apply(updatedFacts : Conjunction, goal : Goal) =
    new UpdateGoalTaskResult(updatedFacts, goal.eliminatedConstants, goal.order, List())
    
  def apply(newTasks : Iterable[Task], goal : Goal) =
    new UpdateGoalTaskResult(goal.facts, goal.eliminatedConstants, goal.order, newTasks)

}

class UpdateGoalTaskResult private (updatedFacts : Conjunction,
                                    updatedElimConstants : Set[ConstantTerm],
                                    updatedOrder : TermOrder,
                                    newTasks : Iterable[Task])
      extends TaskResult {
  def update(goal : Goal) = {
    val updatedTasks = goal.tasks ++ newTasks
    Goal(updatedFacts, updatedTasks, goal.age + 1, updatedElimConstants, updatedOrder)
  }
  
  def updateAndRemoveTask(goal : Goal) = {
    val updatedTasks = goal.tasks.removeFirst ++ newTasks
    Goal(updatedFacts, updatedTasks, goal.age + 1, updatedElimConstants, updatedOrder)    
  }
  
  def toTree(originalGoal : Goal) : ProofTree = update(originalGoal)
}

